#!/usr/bin/python
# -*- coding: utf-8 -*-
import socket
from smtplib import SMTP
from smtplib import SMTPException
import warnings

from flask_mailer.backends.base import Mailer
from flask_mailer.compat import text_type


class SMTPMailer(Mailer):
    """SMTP email backend."""
    def __init__(self,
                 host='localhost',
                 port=25,
                 username=None,
                 password=None,
                 default_sender=None,
                 use_tls=False,
                 **kwargs):
        self.host = host
        self.port = port
        self.use_tls = use_tls
        self.username = username
        self.password = password
        self.default_sender = default_sender

        credentials = (username, password)
        if any(credentials) and not all(credentials):
            warnings.warn(
                'Invalid credentials. Please setup both username and '
                'password or neither.'
            )
            self.username = self.password = None

    def __enter__(self):
        """Acquires the connection to SMTP server."""
        try:
            connection = self.connection = SMTP(self.host, self.port)
            if self.username and self.password:
                connection.login(self.username, self.password)
            if self.use_tls:
                connection.ehlo()
                connection.starttls()
                connection.ehlo()
        except (SMTPException, socket.error) as e:
            raise RuntimeError(str(e))
        return connection

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Releases the exciting connection to SMTP server."""
        try:
            try:
                self.connection.quit()
            except socket.sslerror:
                # This happens when calling quit() on a TLS connection
                # sometimes.
                self.connection.close()
            except:
                return False
        finally:
            self.connection = None
            return True

    def send(self, message):
        """Send the message."""
        with self as con:
            message.from_addr = message.from_addr or self.default_sender
            con.sendmail(text_type(message.from_addr),
                         text_type(message.send_to),
                         message.format())

    def send_quiet(self, message):
        """Send the message but swallow exceptions."""
        try:
            return self.send(message)
        except Exception:
            return


if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/python
# -*- coding: utf-8 -*-
from flask_mailer.backends.base import Mailer


class DummyMailer(Mailer):
    """Dummy mailing instance for unittests.

    Keeps all sent messages in *oubox* list.
    """
    def __init__(self, **kwargs):
        self.outbox = []

    def send(self, message):
        """Sending a message to the dummy *outbox*."""
        self.outbox.append(message)

    def send_quiet(self, message):
        """Actually don't swallow exception."""
        self.send(message)



#!/usr/bin/python
# -*- config coding: utf-8 -*-
from werkzeug.utils import import_string

from flask_mailer.compat import iteritems


def key(name): return ('MAILER_%s' % name).upper()
"""Returns uppercased config key for extension."""


def strip_prefix(prefix, string):
    """Strip prefix from a string if it exists.

    :param prefix: The prefix to strip.
    :param string: The string to process.
    """
    if string.startswith(prefix):
        return string[len(prefix):]
    return string


def get_config(config):
    """Returns the config without the annoying prefix and with lowercase keys.

    :param config: The config dictionary to inspect.
    """
    prefix = 'MAILER_'
    return dict(
        (strip_prefix(prefix, name).lower(), value)
        for name, value in iteritems(config) if name.startswith(prefix)
    )


def import_path(path):
    """"Imports a dotted module path and returns the attribute/class designated
    by the last name in the path.

    :param path: The dotted module path to import.
    """
    if not path:
        return

    module_name, class_name = path.rsplit('.', 1)
    module = import_string(module_name, silent=True)
    return getattr(module, class_name, None)



#!/usr/bin/python
# -*- coding: utf-8 -*-
from email.mime.text import MIMEText
from email.header import Header
from email.utils import parseaddr
from email.utils import formataddr

from flask_mailer.compat import string_types, text_type, unicode_compatible


def to_list(el):
    """Force convert element to list."""
    if isinstance(el, string_types):
        el = [el,]
    return el


def utf8(s):
    return s.encode('utf-8') if isinstance(s, text_type) else s


def contains_nonascii_characters(raw):
    return not all(ord(c) < 128 for c in raw)


def rfc_compliant(s, encoding):
    """Encode a header string into RFC-compliant format. Do not modify
    string if is contains only ascii letters.
    """
    if contains_nonascii_characters(s):
        return Header(s, encoding).encode()
    return s


def sanitize_address(addr, encoding='utf-8'):
    """Sanitize email address into RFC 2822-compliant string.

    Adopted version from Django mail package
    (https://github.com/django/django/blob/master/django/core/mail/message.py).

    :param addr: The address to process.
    :param encoding: The character set that the address was encoded in.
    """
    if isinstance(addr, string_types):
        addr = parseaddr(addr)
    nm, addr = addr

    try:
        nm = rfc_compliant(nm, encoding)
    except UnicodeEncodeError:
        nm = rfc_compliant(nm, 'utf-8')

    try:
        addr.encode('ascii')
    except UnicodeEncodeError:
        if '@' in addr:
            localpart, domain = addr.split('@', 1)
            localpart = rfc_compliant(localpart, encoding)
            domain = domain.encode('idna').decode('ascii')
            addr = '@'.join([localpart, domain])
        else:
            addr = rfc_compliant(addr, encoding)

    return ''.join(formataddr((nm, addr)).splitlines())


class Proxy(object):
    """Create a proxy descriptor.

    Descriptor uses to transparently converts value to given type.

    :param type: The type to converts to
    :param attribute_name: The name of the attribute to store the converted value
    """

    def __init__(self, type, attribute_name):
        self.type = type
        self.attribute_name = attribute_name

    def __get__(self, instance, owner):
        return getattr(instance, self.attribute_name, None)

    def __set__(self, instance, value):
        if not isinstance(value, self.type):
            value = self.type(value)
        setattr(instance, self.attribute_name, value)

    def __delete__(self, instance):
        setattr(instance, self.attribute_name, None)


@unicode_compatible
class SafeHeader(object):
    """A wrapper for RFC 2822-compliant header.

    >>> str(SafeHeader('Hello!'))
    'Hello!'

    Strips any newline characters to prevent header injection::

    >>> str(SafeHeader('No \\n\\rmore header injection!'))
    'No more header injection!'

    Encode string if it contains nonascii characters::

    >>> str(SafeHeader(u'Привет', encoding='cp1251'))
    '=?cp1251?b?z/Do4uXy?='

    :param value: The initial header value.
    :param encoding: The character set that the header was encoded in.
    """
    def __init__(self, value='', encoding='utf-8'):
        self.value = value
        self.encoding = encoding

    def __str__(self):
        if isinstance(self.value, string_types):
            return rfc_compliant(''.join(self.value.splitlines()), self.encoding)
        return text_type(self.value)

    def __nonzero__(self):
        return isinstance(self.value, string_types) and bool(self.value)


@unicode_compatible
class Address(object):
    """A wrapper for email address.

    Perform sanitizing and formating email address. Formated address
    RFC 2822-compliant and suitable to use in internationalized email headers::

    >>> str(Address(u'álice@example.com'))
    '=?utf-8?b?w4PCoWxpY2U=?=@example.com'

    If address consists of two-element list, they handled as the name, email
    address pair::

    >>> str(Address(('Alice', 'alice@example.com')))
    'Alice <alice@example.com>'

    :param address: The email address
    """

    def __init__(self, address):
        self.address = address

    def __str__(self):
        return sanitize_address(self.address)

    def __nonzero__(self):
        return bool(self.address)

    def __eq__(self, obj):
        if isinstance(obj, Address):
            return text_type(self) == text_type(obj)
        elif isinstance(obj, string_types):
            return text_type(self) == obj
        elif isinstance(obj, (list, tuple)):
            return text_type(self) == Address(obj)
        raise NotImplementedError(
            'Unable to compare Address instance against '
            '{} instance'.format(type(obj))
        )

    def __ne__(self, obj):
        return not self == obj

    def __len__(self):
        return len(text_type(self))


@unicode_compatible
class Addresses(list):
    """A base class for email address list.

    Transparently converts appended values to :class:`Address` instances.
    Formated list is suitable to use in mail header::

    >>> cc = Addresses(('Alice', 'alice@example.com'))
    >>> cc.append(('Bob', 'bob@example.com'))
    >>> str(cc)
    'Alice <alice@example.com>, Bob <bob@example.com>'

    """

    def __init__(self, values=None):
        super(Addresses, self).__init__()

        if values is None:
            return
        elif isinstance(values, tuple):
            self.append(values)
            return
        elif isinstance(values, string_types):
            values = values,
        elif isinstance(values, Address):
            values = values,

        self.extend(values)

    def __str__(self):
        return ', '.join(map(text_type, self))

    def append(self, value):
        return self.extend([value,])

    def extend(self, iterable):
        """Extend list by appending elements from the iterable.

        Convert each value to :class:`Address` instance before extend.
        """
        values = [Address(x) if not isinstance(x, Address) else x
                  for x in iterable]
        super(Addresses, self).extend(values)


@unicode_compatible
class Email(object):
    """Base class for email messages.

    >>> mail = Email(
    ...    subject='hello, there',
    ...    text='awesome message',
    ...    to=['to@example.com', 'you@example.com'],
    ...    from_addr='me@example.com'
    ... )
    >>> message = mail.to_message()
    >>> message['From']
    u'me@example.com'
    >>> message['To']
    u'to@example.com, you@example.com'
    >>> message['Subject']
    u'hello, there'

    """

    subject = Proxy(SafeHeader, '_subject')
    from_addr = Proxy(Address, '_from_addr')
    reply_to = Proxy(Address, '_reply_to')
    bcc = Proxy(Addresses, '_bcc')
    to = Proxy(Addresses, '_to')
    cc = Proxy(Addresses, '_cc')

    def __init__(self,
                 subject=None,
                 text=None,
                 to=None,
                 from_addr=None,
                 cc=None,
                 bcc=None,
                 reply_to=None):
        self.text = text
        self.subject = subject
        self.from_addr = from_addr
        self.to = to
        self.cc = cc
        self.bcc = bcc
        self.reply_to = reply_to

    @property
    def send_to(self):
        """Returns list of unique recipients of the email. List includes direct
        addressees as well as Cc and Bcc entries.
        """
        to = map(text_type, self.to)
        cc = map(text_type, self.cc)
        bcc = map(text_type, self.bcc)
        uniq = set(to) | set(cc) | set(bcc)
        return Addresses(uniq)

    def to_message(self):
        """Returns the email as MIMEText object."""
        if not self.text or not self.subject or \
           not self.to or not self.from_addr:
            raise ValueError('Fill in mailing parameters first')

        msg = MIMEText(utf8(self.text))

        # really MIMEText is sucks, it does not override values on setitem,
        # it appends them. Remove some predefined fields
        del msg['Content-Type']
        del msg['Content-Transfer-Encoding']

        msg['From'] = text_type(self.from_addr)
        msg['To'] = text_type(self.to)
        msg['Subject'] = text_type(self.subject)
        msg['Content-Type'] = 'text/plain; charset=utf-8'
        msg['Content-Transfer-Encoding'] = '8bit'

        if self.cc:
            msg['Cc'] = text_type(self.cc)

        if self.reply_to:
            msg['Reply-To'] = text_type(self.reply_to)

        return msg

    def format(self, sep='\r\n'):
        """Format message into a string."""
        return sep.join(self.to_message().as_string().splitlines())

    def __str__(self):
        return self.format(sep='\n')


if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys


if sys.version_info[0] >= 3:
    text_type = str
    string_types = str,
    iteritems = lambda o: o.items()
    itervalues = lambda o: o.values()

    unicode_compatible = lambda x: x
else:
    text_type = unicode
    string_types = basestring,
    iteritems = lambda o: o.iteritems()
    itervalues = lambda o: o.itervalues()

    def unicode_compatible(cls):
        """A decorator which defines `__str__` and `__unicode__` methods in
        decorated class.
        """
        if '__str__' not in cls.__dict__:
            raise ValueError(
                'decorator cannot be applied to %s '
                'because it does not define __str__ method' % cls.__name__
            )
        cls.__unicode__ = cls.__str__
        cls.__str__ = lambda x: x.__unicode__().encode('utf-8')
        return cls



#!/usr/bin/python
# -*- coding: utf-8 -*-


class Mailer(object):
    """Base email backend manager."""

    def __init__(self, **kwargs):
        pass

    def send(self, message):
        """Send the email message."""
        raise NotImplementedError

    def send_quiet(self, message):
        """Send the message but swallow exceptions."""
        raise NotImplementedError



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Flask-Mailer
============

A Flask extension for sending email messages with pluggable backends
and pythonic API.

Contributing
------------

Don't hesitate to create a `GitHub issue
<https://github.com/vitalk/flask-mailer/issues>`_ for any **bug** or
**suggestion**.

"""
import io
import os
import re
import sys
import subprocess
from setuptools import find_packages, setup, Command


def read(*parts):
    try:
        return io.open(os.path.join(*parts), 'r', encoding='utf-8').read()
    except IOError:
        return b''


def get_version():
    version_file = read('flask_mailer', '__init__.py')
    version_match = re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]',
                              version_file, re.MULTILINE)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


class pytest(Command):
    user_options = [
        ('coverage', None, 'report coverage')
    ]

    def initialize_options(self):
        self.coverage = None

    def finalize_options(self):
        pass

    def run(self):
        basecmd = [sys.executable, '-m', 'pytest']
        if self.coverage:
            basecmd += ['--cov', 'flask_mailer']
        errno = subprocess.call(basecmd + ['tests'])
        raise SystemExit(errno)


__version__ = get_version()


setup(
    name='Flask-Mailer',
    version=__version__,
    license='BSD',
    author='Vital Kudzelka',
    author_email='vital.kudzelka@gmail.com',
    description='A Flask extension for sending emails with pluggable backends.',
    url='https://github.com/vitalk/flask-mailer',
    download_url='https://github.com/vitalk/flask-mailer/tarball/%s' % __version__,
    long_description=__doc__,
    packages=find_packages(exclude=['tests']),
    install_requires=['Flask'],
    tests_require=['pytest', 'pytest-cov'],
    cmdclass={'test': pytest},
    zip_safe=False,
    platforms='any',
    keywords='flask mail smtp',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



#!/usr/bin/python
# -*- coding: utf-8 -*-
from flask import current_app

from flask_mailer.mail import Email
from flask_mailer.util import key
from flask_mailer.util import get_config
from flask_mailer.util import import_path


__version__ = '0.4.0'
__all__ = ('send_email', 'Mailer', 'Email')


def send_email(subject, text, to, fail_quiet=True):
    """Send an email."""
    mailer = _get_mailer()
    mail = Email(subject, text, to)
    if fail_quiet:
        return mailer.send_quiet(mail)
    return mailer.send(mail)


def _get_mailer(state=None):
    """Try to get mailer instance registered to with current application and
    raise error otherwise.
    """
    app = getattr(state, 'app', None) or current_app

    if not hasattr(app, 'extensions') or \
       'mailer' not in app.extensions:
           raise RuntimeError(
                'Mailer extension does not registered with current application'
                'or no application bound to current request'
           )

    return app.extensions['mailer']


class Mailer(object):
    """Mailer instance manages sending of email messages."""

    def __init__(self, app=None):
        self.app = app
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        config = app.config
        config.setdefault(key('testing'), app.testing)
        config.setdefault(key('host'), 'localhost')
        config.setdefault(key('port'), 25)
        config.setdefault(key('use_tls'), False)
        config.setdefault(key('username'), None)
        config.setdefault(key('password'), None)
        config.setdefault(key('default_sender'), 'webmaster')
        config.setdefault(key('backend'), 'flask_mailer.backends.smtp.SMTPMailer')

        # Use dummy mailer for testing application.
        if config[key('testing')]:
            config[key('backend')] = 'flask_mailer.backends.dummy.DummyMailer'

        state = self.init_backend(config)

        # Register extension themselves for backwards compatibility.
        app.extensions = getattr(app, 'extensions', {})
        app.extensions['mailer'] = state
        return state

    def init_backend(self, options=None):
        """Use options to create a new mailer backend."""
        options = get_config(options or {})

        backend_path = options.pop('backend', None)
        backend_class = import_path(backend_path)
        if backend_class is None:
            raise RuntimeError("Invalid backend: '%s'" % backend_path)

        return backend_class(**options)

    def __getattr__(self, name):
        return getattr(_get_mailer(self), name, None)


if __name__ == '__main__':
    import doctest
    doctest.testmod()
